# Antivir scanners

from basic import *
from dazuko import *
from clamav import *
from avg import avgd
from bdc import bdc
from nod import *
from sophie import sophie
from kav import *
from icap import *
from sanitize import *
